@@ -1,3 +1,53 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Command::Depotmap;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
@@ -6,7 +56,7 @@ use base qw( SVK::Command );
 use SVK::XD;
 use SVK::I18N;
 use SVK::Util qw( get_buffer_from_editor abs_path move_path );
-use YAML;
+use YAML::Syck;
 use File::Path;
 
 sub options {
@@ -25,10 +75,11 @@ sub parse_arg {
     return undef
 	unless $self->{add} or $self->{detach} or $self->{relocate};
 
-        @arg or die loc("Need to specify a depot name");
+        @arg or die loc("Need to specify a depot name.\n");
 
         my $depot = shift(@arg);
-        @arg or die loc("Need to specify a path name") unless $self->{detach};
+        @arg or die loc("Need to specify a path name for depot.\n")
+            unless $self->{detach};
 
         my $map = $self->{xd}{depotmap};
         my $path = $depot;
@@ -50,14 +101,14 @@ sub parse_arg {
 sub run {
     my ($self) = @_;
     my $sep = '===edit the above depot map===';
-    my $map = YAML::Dump ($self->{xd}{depotmap});
+    my $map = YAML::Syck::Dump ($self->{xd}{depotmap});
     my $new;
     if ( !$self->{'init'} ) {
         do {
             $map =
               get_buffer_from_editor( loc('depot map'), $sep, "$map\n$sep\n",
                 'depotmap' );
-            $new = eval { YAML::Load($map) };
+            $new = eval { YAML::Syck::Load($map) };
             print "$@\n" if $@;
         } while ($@);
         print loc("New depot map saved.\n");
@@ -168,17 +219,3 @@ The depotname may then be used as part of a DEPOTPATH:
 Depot creation respects $ENV{SVNFSTYPE}, which is default to fsfs for
 svn 1.1 or later, and bdb for svn 1.0.x.
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
