@@ -1,10 +1,60 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Command::Mirror;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command::Commit );
 use SVK::I18N;
-use SVK::Util qw( HAS_SVN_MIRROR is_uri get_prompt traverse_history );
+use SVK::Util qw( is_uri get_prompt traverse_history );
 
 use constant narg => undef;
 
@@ -21,7 +71,6 @@ sub lock {} # override commit's locking
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
 
     @arg = ('//') if $self->{upgrade} and !@arg;
     return if !@arg;
@@ -41,25 +90,21 @@ sub parse_arg {
 }
 
 sub run {
-    my ($self, $target, $source, @options) = @_;
-    die loc ("%1 already exists.\n", $target->path)
-	if $target->root->check_path ($target->path);
-    $self->get_mirror_object ($target, $source, @options)->init
-	or die loc("%1 already mirrored, use 'svk mirror --detach' to remove it first.\n",
-		   $target->{depotpath});
-    return;
-}
+    my ( $self, $target, $source, @options ) = @_;
 
-sub get_mirror_object {
-    my ($self, $target, $source, @options) = @_;
-    return SVN::Mirror->new (target_path => $target->{path},
-			     source => $source,
-			     repospath => $target->{repospath},
-			     repos => $target->{repos},
-			     options => \@options,
-			     config => $self->{svnconfig},
-			     pool => SVN::Pool->new,
-			    );
+    SVK::Mirror->create(
+        {
+            depot   => $target->depot,
+            path    => $target->path,
+            backend => 'SVNRa',
+            url     => "$source", # this can be an URI object
+            pool    => SVN::Pool->new
+        }
+    );
+
+    print loc("Mirror initialized.  Run svk sync %1 to start mirroring.\n", $target->report);
+
+    return;
 }
 
 package SVK::Command::Mirror::relocate;
@@ -68,7 +113,8 @@ use SVK::I18N;
 
 sub run {
     my ($self, $target, $source, @options) = @_;
-    $self->get_mirror_object ($target, $source, @options)->relocate;
+    $target->is_mirrored->relocate($source, @options);
+    print loc("Mirror relocated.\n");
     return;
 }
 
@@ -80,14 +126,13 @@ use constant narg => 1;
 
 sub run {
     my ($self, $target) = @_;
-    my ($m, $mpath) = SVN::Mirror::is_mirrored ($target->{repos},
-						$target->{path});
+    my ($m, $mpath) = $target->is_mirrored;
 
-    die loc("%1 is not a mirrored path.\n", $target->{depotpath}) if !$m;
-    die loc("%1 is inside a mirrored path.\n", $target->{depotpath}) if $mpath;
+    die loc("%1 is not a mirrored path.\n", $target->depotpath) if !$m;
+    die loc("%1 is inside a mirrored path.\n", $target->depotpath) if $mpath;
 
-    $m->delete(1); # remove svm:source and svm:uuid too
-    print loc("Mirror path '%1' detached.\n", $target->{depotpath});
+    $m->detach(1); # remove svm:source and svm:uuid too
+    print loc("Mirror path '%1' detached.\n", $target->depotpath);
     return;
 }
 
@@ -99,7 +144,7 @@ use constant narg => 1;
 
 sub run {
     my ($self, $target) = @_;
-    SVN::Mirror::upgrade ($target->{repos});
+    print loc("nothing to upgrade\n");
     return;
 }
 
@@ -111,57 +156,54 @@ use constant narg => 1;
 
 sub run {
     my ($self, $target) = @_;
-    my $m = SVN::Mirror->new
-	( target_path => $target->path,
-	  repos => $target->{repos},
-	  ignore_lock => 1,
-	  get_source => 1
-	);
-    $m->init;
-    $m->unlock ('force');
-    print loc ("mirror locks on %1 removed.\n", $target->{report});
+    $target->depot->mirror->unlock($target->path_anchor);
+    print loc ("mirror locks on %1 removed.\n", $target->report);
     return;
 }
 
 package SVK::Command::Mirror::list;
-use SVK::Util qw( HAS_SVN_MIRROR );
 use base qw(SVK::Command::Mirror);
 use SVK::I18N;
+use List::Util qw( max );
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
     return (@arg ? @arg : undef);
 }
 
 sub run {
-    my ($self, $target) = @_;
-    my $fmt = "%-20s\t%-s\n";
-    printf $fmt, loc('Path'), loc('Source');
-    print '=' x 60, "\n";
-    my @depots = (defined($_[1])) ? @_[1..$#_] : sort keys %{$self->{xd}{depotmap}};
+    my ( $self, $target ) = @_;
+
+    my @mirror_columns;
+    my @depots
+        = defined $target
+        ? @_[ 1 .. $#_ ]
+        : sort keys %{ $self->{xd}{depotmap} }
+        ;
+    DEPOT:
     foreach my $depot (@depots) {
-	$depot =~ s{/}{}g;
-	$target = eval { $self->arg_depotpath ("/$depot/") };
-	if ($@) {
-	    warn loc ("Depot /%1/ not loadable.\n", $depot);
-	    next;
-	}
-	my @paths = SVN::Mirror::list_mirror ($target->{repos});
-	my $fs = $target->{repos}->fs;
-	my $root = $fs->revision_root ($fs->youngest_rev);
-	my $name = $target->depotname;
-	foreach my $path (@paths) {
-	    eval {
-	    my $m = SVN::Mirror->new(
-                    target_path => $path,
-                    repos => $target->{repos},
-                    get_source => 1
-                );
-	    printf $fmt, "/$name$path", $m->{source};
-	    };
-	}
+        $depot =~ s{/}{}g;
+        $target = eval { $self->arg_depotpath("/$depot/") };
+        if ($@) {
+            warn loc( "Depot /%1/ not loadable.\n", $depot );
+            next DEPOT;
+        }
+        my $depot_name = $target->depotname;
+        foreach my $path ( $target->depot->mirror->entries ) {
+            my $m = $target->depot->mirror->get($path);
+            push @mirror_columns, [ "/$depot_name$path", $m->url ];
+        }
     }
+
+    my $max_depot_path = max map { length $_->[0] } @mirror_columns;
+    my $max_uri        = max map { length $_->[1] } @mirror_columns;
+
+    my $fmt = "%-${max_depot_path}s   %-s\n";
+    printf $fmt, loc('Path'), loc('Source');
+    print '=' x ( $max_depot_path + $max_uri + 3 ), "\n";
+
+    printf $fmt, @$_ for @mirror_columns;
+
     return;
 }
 
@@ -169,13 +211,15 @@ package SVK::Command::Mirror::recover;
 use base qw(SVK::Command::Mirror);
 use SVK::Util qw( traverse_history get_prompt );
 use SVK::I18N;
+use SVK::Logger;
 
 use constant narg => 1;
 
 sub run {
     my ($self, $target, $source, @options) = @_;
-    $source = ("file://$target->{repospath}");
-    my $m = $self->get_mirror_object ($target, $source, @options);
+    die loc("recover not supported.\n");
+    my ($m, $mpath) = $target->is_mirrored;
+
     $self->recover_headrev ($target, $m);
     $self->recover_list_entry ($target, $m);
     return;
@@ -184,7 +228,7 @@ sub run {
 sub recover_headrev {
     my ($self, $target, $m) = @_;
 
-    my $fs = $m->{fs};
+    my $fs = $target->repos->fs;
     my ($props, $headrev, $rev, $firstrev, $skipped, $uuid, $rrev);
 
     traverse_history (
@@ -194,9 +238,9 @@ sub recover_headrev {
         callback    => sub {
             $rev = $_[1];
             $firstrev ||= $rev;
-            print loc("Analyzing revision %1...\n", $rev),
+            $logger->info(loc("Analyzing revision %1...\n", $rev),
                   ('-' x 70),"\n",
-                  $fs->revision_prop ($rev, 'svn:log'), "\n";
+                  $fs->revision_prop ($rev, 'svn:log'));
 
             if ( $headrev = $fs->revision_prop ($rev, 'svm:headrev') ) {
                 ($uuid, $rrev) = split(/[:\n]/, $headrev);
@@ -217,7 +261,7 @@ sub recover_headrev {
     }
 
     if (!$skipped) {
-        print loc("No need to revert; it is already the head revision.\n");
+        $logger->warn(loc("No need to revert; it is already the head revision."));
         return;
     }
 
@@ -230,9 +274,10 @@ sub recover_headrev {
         delete => { direct => 1, message => '' }
     )->run($target);
 
+    $target->refresh_revision;
     $self->command(
-        copy => { rev => $rev, direct  => 1, message => '' },
-    )->run($target => $target);
+        copy => { direct  => 1, message => '' },
+    )->run($target->new(revision => $rev) => $target->new);
 
     # XXX - race condition? should get the last committed rev instead
     $target->refresh_revision;
@@ -248,7 +293,7 @@ sub recover_headrev {
 sub recover_list_entry {
     my ($self, $target, $m) = @_;
 
-    my %mirrors = map { ($_ => 1) } SVN::Mirror::list_mirror ($target->{repos});
+    my %mirrors = map { ($_ => 1) } SVN::Mirror::list_mirror ($target->repos);
 
     return if $mirrors{$m->{target_path}}++;
 
@@ -257,7 +302,7 @@ sub recover_list_entry {
         $self->arg_depotpath ('/'.$target->depotname.'/'),
     );
 
-    print loc("%1 added back to the list of mirrored paths.\n", $target->{report}); 
+    print loc("%1 added back to the list of mirrored paths.\n", $target->report);
     return;
 }
 
@@ -290,22 +335,8 @@ SVK::Command::Mirror - Initialize a mirrored depotpath
 
  -l [--list]            : list mirrored paths
  -d [--detach]          : mark a depotpath as no longer mirrored
- --relocate             : relocate the mirror to another URI
+ --relocate             : change the upstream URI for the mirrored depotpath
  --recover              : recover the state of a mirror path
  --unlock               : forcibly remove stalled locks on a mirror
  --upgrade              : upgrade mirror state to the latest version
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
