@@ -1,3 +1,53 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Command::Status;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
@@ -26,7 +76,8 @@ sub flush_print {
     my ($crev, $author);
     my $fs = $root->fs;
     if ($from_path) {
-	$from_path =~ s{^file://\Q$target->{repospath}\E}{};
+	my $repospath = $target->repospath;
+	$from_path =~ s{^file://\Q$repospath\E}{};
         $crev = $fs->revision_root ($from_rev)->node_created_rev ($from_path);
 	$author = $fs->revision_prop ($crev, 'svn:author');
 	$baserev = '-';
@@ -37,16 +88,16 @@ sub flush_print {
     } elsif ($status->[0] eq 'A') {
 	$baserev = 0;
     } elsif ($status->[0] !~ '[!~]') {
-        my $p = $target->{path};
+        my $p = $target->path_anchor;
 	my $path = $p eq '/' ? "/$entry" : (length $entry ? "$p/$entry" : $p);
 	$crev = $root->node_created_rev ($path);
 	$author = $fs->revision_prop($crev, 'svn:author') unless $crev == -1;
     }
 
-    my $report = $target->{report};
+    my $report = $target->report;
     my $newentry = length $entry
-	? SVK::Target->copath ($report, $entry)
-	: SVK::Target->copath ('', length $report ? $report : '.');
+	? SVK::Path::Checkout->copath ($report, $entry)
+	: SVK::Path::Checkout->copath ('', length $report ? $report : '.');
     no warnings 'uninitialized';
     print sprintf ("%1s%1s%1s %8s %8s %-12.12s \%s\n", @{$status}[0..2],
                    defined $baserev ? $baserev : '? ',
@@ -57,43 +108,39 @@ sub flush_print {
 
 sub run {
     my ($self, $target) = @_;
-    my $xdroot = $self->{xd}->xdroot (%$target);
-    my $report = $target->{report};
-    $report .= '/' if $report;
     my $editor = SVK::Editor::Status->new (
-	  report => $target->{report},
 	  ignore_absent => $self->{quiet},
 	  $self->{verbose} ?
 	  (notify => SVK::Notify->new (
 	       flush_baserev => 1,
 	       flush_unchanged => 1,
-	       cb_flush => sub { flush_print ($xdroot, $target, @_); }
+	       cb_flush => sub { flush_print ($target->root, $target, @_); }
 	   )
 	  )                :
-	  (notify => SVK::Notify->new_with_report ($target->{report},
+	  (notify => SVK::Notify->new_with_report ($target->report,
 		undef, 1)
 	  )
       );
     $self->{xd}->checkout_delta
-	( %$target,
-	  xdroot => $xdroot,
+	( $target->for_checkout_delta,
+	  xdroot => $target->create_xd_root,
 	  nodelay => 1,
 	  delete_verbose => 1,
 	  editor => $editor,
-	  cb_conflict => \&SVK::Editor::Status::conflict,
-	  cb_obstruct => \&SVK::Editor::Status::obstruct,
+	  cb_conflict => sub { shift->conflict(@_) },
+	  cb_obstruct => sub { shift->obstruct(@_) },
 	  $self->{verbose} ?
-	      (cb_unchanged => \&SVK::Editor::Status::unchanged
+	      (cb_unchanged => sub { shift->unchanged(@_) },
 	      )            :
 	      (),
 	  $self->{recursive} ? () : (depth => 1),
 	  $self->{no_ignore} ?
-              (cb_ignored => \&SVK::Editor::Status::ignored
+              (cb_ignored => sub { shift->ignored(@_) },
               )              :
               (),
 	  $self->{quiet} ?
               ()         :
-              (cb_unknown => \&SVK::Editor::Status::unknown)
+              (cb_unknown => sub { shift->unknown(@_) } )
 	);
     return;
 }
@@ -165,17 +212,3 @@ Example output:
             53       43 harry        wc/zig.c
             53       20 sally        wc
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
