@@ -1,11 +1,67 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Editor::Merge;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
-require SVN::Delta;
-our @ISA = qw(SVN::Delta::Editor);
+use base 'SVK::Editor';
 use SVK::I18N;
-use autouse 'SVK::Util' => qw( slurp_fh md5_fh get_anchor tmpfile devnull );
+use SVK::Logger;
+use autouse 'SVK::Util'
+    => qw( slurp_fh md5_fh tmpfile devnull abs2rel );
+
+__PACKAGE__->mk_accessors(qw(inspector notify storage));
+
+use Class::Autouse qw(SVK::Inspector::Root SVK::Notify
+		      Data::Hierarchy IO::Digest);
 
 use constant FH => 0;
 use constant FILENAME => 1;
@@ -70,6 +126,10 @@ open the directory even if cb_exist failed. This is for use in
 conjunction with L<SVK::Editor::Rename> for the case that a descendent
 exists but its parent does not.
 
+=item inspector
+
+The inspector reflecting the target of the merge.
+
 =back
 
 =head2 callbacks for local tree
@@ -79,10 +139,6 @@ callbacks must be supplied.
 
 =over
 
-=item cb_exist
-
-Check if the given path exists.
-
 =item cb_rev
 
 Check the revision of the given path.
@@ -91,29 +147,11 @@ Check the revision of the given path.
 
 Called when a conflict is detected.
 
-=item cb_localmod
-
-Called when the merger needs to retrieve the local modification of a
-file. Return an arrayref of filename, filehandle, and md5. Return
-undef if there is no local modification.
-
-=item cb_localprop
-
-Called when the merger needs to retrieve the local modification of a
-property. Return the property value.
-
 =item cb_prop_merged
 
 Called when properties are merged without changes, that is, the C<g>
 status.
 
-=item cb_dirdelta
-
-When C<delete_entry> needs to check if everything to be deleted does
-not cause conflict on the directory, it calls the callback with path,
-base_root, and base_path. The returned value should be a hash with
-changed paths being the keys and change types being the values.
-
 =item cb_merged
 
 Called right before closing the top directory with storage editor,
@@ -130,58 +168,59 @@ Called after each file close call.
 use Digest::MD5 qw(md5_hex);
 use File::Compare ();
 
-sub cb_for_root {
-    my ($class, $root, $anchor, $base_rev) = @_;
-    return ( cb_exist =>
-	     sub { my ($path, $pool) = @_;
-		   $path = $anchor.'/'.$path;
-		   return $root->check_path ($path, $pool);
-	       },
-	     cb_rev => sub { $base_rev; },
-	     cb_localmod =>
-	     sub { my ($path, $checksum, $pool) = @_;
-		   $path = "$anchor/$path";
-		   my $md5 = $root->file_md5_checksum ($path, $pool);
-		   return if $md5 eq $checksum;
-		   return [$root->file_contents ($path, $pool),
-			   undef, $md5];
-	       },
-	     cb_localprop =>
-	     sub { my ($path, $propname, $pool) = @_;
-		   $path = "$anchor/$path";
-		   local $@;
-		   return eval { $root->node_prop ($path, $propname, $pool) };
-	       },
-	     cb_dirdelta =>
-	     sub { my ($path, $base_root, $base_path, $pool) = @_;
-		   my $modified;
-		   my $editor =  SVK::Editor::Status->new
-		       ( notify => SVK::Notify->new
-			 ( cb_flush => sub {
-			       my ($path, $status) = @_;
-			       $modified->{$path} = $status->[0];
-			   }));
-		   SVK::XD->depot_delta (oldroot => $base_root, newroot => $root,
-					 oldpath => [$base_path, ''],
-					 newpath => "$anchor/$path",
-					 editor => $editor,
-					 no_textdelta => 1, no_recurse => 1);
-		   return $modified;
-	       },
-	   );
+sub new {
+    my $class = shift;
+
+    my $self = $class->SUPER::new(ref $_[0] ? @_ :{@_});
+
+    if ($self->storage->can('rename_check')) {
+	my $editor = $self->storage;
+	$self->inspector_translate
+	    (sub { $_[0] = $editor->rename_check($_[0])});
+
+	my $flush = $self->notify->{cb_flush};
+	$self->notify->{cb_flush} = sub {
+	    my ($path, $st) = @_;
+	    my $newpath = $self->storage->rename_check($path);
+	    $flush->($path, $st, $path eq $newpath ? undef : $newpath) };
+    }
+
+    return $self;
 }
 
-# translate the path before passing to cb_*
-sub cb_translate {
-    my ($cb, $translate) = @_;
-    for (qw/cb_exist cb_rev cb_conflict cb_localmod cb_localprop cb_dirdelta/) {
-	my $sub = $cb->{$_};
-	next unless $sub;
-	$cb->{$_} = sub { my $path = shift; $translate->($path);
-			  unshift @_, $path; goto &$sub };
+sub cb_for_root {
+    my ($class, $root, $anchor, $base_rev) = @_;
+    # XXX $root and $anchor are actually SVK::Path
+    my $inspector = SVK::Inspector::Root->new({
+        root => $root, 
+        anchor => $anchor, 
+    });
+
+    return (
+        inspector => $inspector,
+        cb_rev => sub { $base_rev },
+    );
+}
+
+sub inspector_translate {
+    my ($self, $translate) = @_;
+    # XXX: should do a real clone and then push
+    $self->inspector($self->inspector->new($self->inspector));
+    $self->inspector->path_translations([]);
+    $self->inspector->push_translation($translate);
+    for (qw/cb_conflict/) {
+        my $sub = $self->{$_};
+        next unless $sub;
+        $self->{$_} = sub { my $path = shift; $translate->($path);
+			    unshift @_, $path; goto &$sub };
     }
 }
 
+sub copy_info {
+    my ($self, $src_from, $src_fromrev, $dst_from, $dst_fromrev) = @_;
+    $self->{copy_info}{$src_from}{$src_fromrev} = [$dst_from, $dst_fromrev];
+}
+
 sub set_target_revision {
     my ($self, $revision) = @_;
     $self->{revision} = $revision;
@@ -197,6 +236,7 @@ sub open_root {
     $self->{notify}->node_status ('', '');
 
     my $ticket = $self->{ticket};
+    $self->{dh} = Data::Hierarchy->new;
     $self->{cb_merged} =
 	sub { my ($editor, $baton, $type, $pool) = @_;
 	      my $func = "change_${type}_prop";
@@ -213,7 +253,7 @@ sub add_file {
     # a replaced node shouldn't be checked with cb_exist
     my $touched = $self->{notify}->node_status($path);
     if (!$self->{added}{$pdir} && !$touched &&
-	(my $kind = $self->{cb_exist}->($path, $pool))) {
+	(my $kind = $self->inspector->exist($path, $pool))) {
 	unless ($kind == $SVN::Node::file) {
 	    $self->{notify}->flush ($path) ;
 	    return undef;
@@ -221,11 +261,21 @@ sub add_file {
 	$self->{info}{$path}{addmerge} = 1;
 	$self->{info}{$path}{open} = [$pdir, -1];
 	$self->{info}{$path}{fpool} = $pool;
+	if (defined $arg[0]) {
+	    warn "===> add merge with history... very bad";
+	}
+	$self->{cb_add_merged}->($path) if $self->{cb_add_merged};
     }
     else {
 	++$self->{changes};
 	$self->{added}{$path} = 1;
 	$self->{notify}->node_status ($path, $touched ? 'R' : 'A');
+	if (defined $arg[0]) {
+	    $self->{notify}->hist_status ($path, '+');
+	    @arg = $self->resolve_copy($path, @arg);
+	    $self->{info}{$path}{baseinfo} = [$self->_resolve_base($path)];
+	    $self->{info}{$path}{fpool} = $pool;
+	}
 	$self->{storage_baton}{$path} =
 	    $self->{storage}->add_file ($path, $self->{storage_baton}{$pdir}, @arg, $pool);
 	$pool->default if $pool && $pool->can ('default');
@@ -236,10 +286,26 @@ sub add_file {
     return $path;
 }
 
+sub _resolve_base {
+    my ($self, $path, $orig) = @_;
+    my ($entry) = $self->{dh}->get("/$path");
+    return unless $entry->{copyanchor};
+    $entry = $self->{dh}->get($entry->{copyanchor})
+	unless $entry->{copyanchor} eq "/$path";
+    my $key = $orig ? 'orig_copyfrom' : 'copyfrom';
+    return (abs2rel("/$path",
+		    $entry->{copyanchor} => $entry->{".$key"}, '/'),
+	    $entry->{".${key}_rev"});
+}
+
 sub open_file {
     my ($self, $path, $pdir, $rev, $pool) = @_;
     # modified but rm locally - tag for conflict?
-    if ($self->{cb_exist}->($path, $pool)) {
+    my ($basepath, $fromrev) = $self->_resolve_base($path);
+    $basepath = $path unless defined $basepath;
+    if ($self->inspector->exist($basepath, $pool) == $SVN::Node::file) {
+	$self->{info}{$path}{baseinfo} = [$basepath, $fromrev]
+	    if defined $fromrev;
 	$self->{info}{$path}{open} = [$pdir, $rev];
 	$self->{info}{$path}{fpool} = $pool;
 	$self->{notify}->node_status ($path, '');
@@ -324,8 +390,15 @@ sub _retrieve_base
 {
     my ($self, $path, $pool) = @_;
     my @base = tmpfile('base-');
-    $path = "$self->{base_anchor}/$path" if $self->{base_anchor};
-    slurp_fh ($self->{base_root}->file_contents ($path, $pool), $base[FH]);
+
+    my ($basepath, $fromrev) = $self->{info}{$path}{baseinfo} ?
+	$self->_resolve_base($path, 1)
+      : ($path);
+    my $root = $fromrev ? $self->{base_root}->fs->revision_root($fromrev, $pool)
+	: $self->{base_root};
+    $basepath = "$self->{base_anchor}/$path"
+	if $basepath !~ m{^/} && $self->{base_anchor};
+    slurp_fh ($root->file_contents ($basepath, $pool), $base[FH]);
     seek $base[FH], 0, 0;
     return @base;
 }
@@ -335,9 +408,10 @@ sub apply_textdelta {
     return unless $path;
 
     my $info = $self->{info}{$path};
+    my ($basepath, $fromrev) = $info->{baseinfo} ? @{$info->{baseinfo}} : ($path);
     my $fh = $info->{fh} = {};
     if (($pool = $info->{fpool}) &&
-	($fh->{local} = $self->{cb_localmod}->($path, $checksum || '', $pool))) {
+	($fh->{local} = $self->inspector->localmod($basepath, $checksum || '', $pool))) {
 	# retrieve base
 	unless ($info->{addmerge}) {
 	    $fh->{base} = [$self->_retrieve_base($path, $pool)];
@@ -350,8 +424,10 @@ sub apply_textdelta {
 	unless $self->{notify}->node_status ($path);
 
     $self->ensure_open ($path);
+
     my $handle = $self->{storage}->apply_textdelta ($self->{storage_baton}{$path},
 						    $checksum, $pool);
+
     if ($self->{storage_has_unwritable} && !$handle) {
 	delete $self->{notify}{status}{$path};
 	$self->{notify}->flush ($path);
@@ -365,18 +441,25 @@ sub _merge_text_change {
 	(map {$fh->{$_}[FILENAME]} qw/base local new/);
     my $mfh = tmpfile ('merged-');
     my $marker = time.int(rand(100000));
+    my $ylabel
+        = ref($self->{inspector}) eq 'SVK::Inspector::Compat'
+        ? $label
+        : $label . ' (' . $self->{inspector}->{anchor}.')'
+        ;
+    my $tlabel = $label . ' (' . $self->{anchor}.')';
     SVN::Core::diff_file_output_merge
 	    ( $mfh, $diff,
 	      (map {
 		  $fh->{$_}[FILENAME]
 	      } qw/base local new/),
 	      "==== ORIGINAL VERSION $label $marker",
-	      ">>>> YOUR VERSION $label $marker",
+	      ">>>> YOUR VERSION $ylabel $marker",
 	      "<<<< $marker",
-	      "==== THEIR VERSION $label $marker",
+	      "==== THEIR VERSION $tlabel $marker",
 	      1, 0, $pool);
 
     my $conflict = SVN::Core::diff_contains_conflicts ($diff);
+    $conflict ||= $self->{tree_conflict};
     if (my $resolve = $self->{resolve}) {
 	$resolve->run
 	    ( fh              => $fh,
@@ -396,16 +479,27 @@ sub _merge_text_change {
 
 sub _overwrite_local_file {
     my ($self, $fh, $path, $nfh, $pool) = @_;
+    # XXX: document why this is like this
+    my $storagebase = $fh->{local};
+    my $info = $self->{info}{$path};
+    my ($basepath, $fromrev) = $info->{baseinfo} ? @{$info->{baseinfo}} : ($path);
+
+    if ($fromrev) {
+	my $sbroot = $self->{base_root}->fs->revision_root($fromrev, $pool);
+	$storagebase->[FH] = $sbroot->file_contents($basepath, $pool);
+	$storagebase->[CHECKSUM] = $sbroot->file_md5_checksum($basepath, $pool);
+    }
+
     my $handle = $self->{storage}->
-	apply_textdelta ($self->{storage_baton}{$path}, $fh->{local}[CHECKSUM],
-			 $pool);
+	apply_textdelta ($self->{storage_baton}{$path},
+			 $storagebase->[CHECKSUM], $pool);
 
     if ($handle && $#{$handle} >= 0) {
 	if ($self->{send_fulltext}) {
 	    SVN::TxDelta::send_stream ($nfh, @$handle, $pool);
 	}
 	else {
-	    seek $fh->{local}[FH], 0, 0;
+	    seek $storagebase->[FH], 0, 0 unless $fromrev; # don't seek for sb
 	    my $txstream = SVN::TxDelta::new($fh->{local}[FH], $nfh, $pool);
 	    SVN::TxDelta::send_txstream ($txstream, @$handle, $pool);
 	}
@@ -435,21 +529,28 @@ sub close_file {
     my $fh = $info->{fh};
     my $iod;
 
+    my ($basepath, $fromrev) = $info->{baseinfo} ? @{$info->{baseinfo}} : ($path);
     no warnings 'uninitialized';
+    my $storagebase_checksum = $fh->{local}[CHECKSUM];
+    if ($fromrev) {
+	$storagebase_checksum = $self->{base_root}->fs->revision_root
+	    ($fromrev, $pool)->file_md5_checksum($basepath, $pool);
+    }
+
     # let close_directory reports about its children
     if ($info->{fh}{new}) {
 
 	$self->_merge_file_unchanged ($path, $checksum, $pool), return
-	    if $checksum eq $fh->{local}[CHECKSUM];
+	    if $checksum eq $storagebase_checksum;
 
-	my $eol = $self->{cb_localprop}->($path, 'svn:eol-style', $pool);
+	my $eol = $self->inspector->localprop($basepath, 'svn:eol-style', $pool);
 	my $eol_layer = SVK::XD::get_eol_layer({'svn:eol-style' => $eol}, '>');
 	$eol_layer = '' if $eol_layer eq ':raw';
 	$self->prepare_fh ($fh, $eol_layer);
 	# XXX: There used be a case that this explicit comparison is
 	# needed, but i'm not sure anymore.
 	$self->_merge_file_unchanged ($path, $checksum, $pool), return
-	    if File::Compare::compare ($fh->{new}[FILENAME], $fh->{local}[FILENAME]) == 0;
+	    if File::Compare::compare ($fh->{new}[FILENAME], $fh->{local}->[FILENAME]) == 0;
 
 	$self->ensure_open ($path);
         if ($info->{addmerge}) {
@@ -462,7 +563,6 @@ sub close_file {
 	$eol_layer = SVK::XD::get_eol_layer({'svn:eol-style' => $eol}, '<');
 	binmode $mfh, $eol_layer or die $! if $eol_layer;
 
-	require IO::Digest;
 	$iod = IO::Digest->new ($mfh, 'MD5');
 
 	if ($self->_overwrite_local_file ($fh, $path, $mfh, $pool)) {
@@ -471,11 +571,16 @@ sub close_file {
 	}
 	$self->cleanup_fh ($fh);
     }
-    elsif ($info->{fpool} && !$self->{notify}->node_status ($path)) {
-	# open but prop edit only, load local checksum
-	if (my $local = $self->{cb_localmod}->($path, $checksum, $pool)) {
-	    $checksum = $local->[CHECKSUM];
-	    close $local->[FH];
+    elsif ($info->{fpool}) {
+	if (!$self->{notify}->node_status($path) || !exists $fh->{local} ) {
+	    # open but without text edit, load local checksum
+	    if ($basepath ne $path) {
+		$checksum = $self->{base_root}->fs->revision_root($fromrev, $pool)->file_md5_checksum($basepath, $pool);
+	    }
+	    elsif (my $local = $self->inspector->localmod($basepath, $checksum, $pool)) {
+		$checksum = $local->[CHECKSUM];
+		close $local->[FH];
+	    }
 	}
     }
 
@@ -486,31 +591,38 @@ sub close_file {
 sub add_directory {
     my ($self, $path, $pdir, @arg) = @_;
     return undef unless defined $pdir;
-    my $pool = $arg[-1];
+    my $pool = pop @arg;
     my $touched = $self->{notify}->node_status($path);
+    undef $touched if $touched && $touched eq 'C';
     # Don't bother calling cb_exist (which might be expensive if the parent is
     # already added.
     if (!$self->{added}{$pdir} && !$touched &&
-	(my $kind = $self->{cb_exist}->($path, $pool))) {
+	(my $kind = $self->inspector->exist($path, $pool))) {
 	unless ($kind == $SVN::Node::dir) {
 	    $self->{notify}->flush ($path) ;
 	    return undef;
 	}
 	$self->{storage_baton}{$path} =
 	    $self->{storage}->open_directory ($path, $self->{storage_baton}{$pdir},
-					      $self->{cb_rev}->($path), $arg[2]);
+					      $self->{cb_rev}->($path), $pool);
 	$self->{notify}->node_status ($path, 'G');
+	$self->{cb_add_merged}->($path) if $self->{cb_add_merged};
     }
     else {
+	if (defined $arg[0]) {
+	    @arg = $self->resolve_copy($path, @arg);
+	}
 	my $baton =
 	    $self->{storage}->add_directory ($path, $self->{storage_baton}{$pdir},
-					     @arg);
+					     @arg, $pool);
 	unless (defined $baton) {
 	    $self->{notify}->flush ($path);
 	    return undef;
 	}
 	$self->{storage_baton}{$path} = $baton;
 	$self->{added}{$path} = 1;
+	$self->{notify}->hist_status ($path, '+')
+	    if defined $arg[0];
 	$self->{notify}->node_status ($path, $touched ? 'R' : 'A');
 	$self->{notify}->flush ($path, 1);
     }
@@ -518,12 +630,33 @@ sub add_directory {
     return $path;
 }
 
+sub resolve_copy {
+    my ($self, $path, $from, $rev) = @_;
+    die "unknown copy $from $rev for $path"
+	unless exists $self->{copy_info}{$from}{$rev};
+    my ($dstfrom, $dstrev) = @{$self->{copy_info}{$from}{$rev}};
+    $self->{dh}->store("/$path", { copyanchor => "/$path",
+				   '.copyfrom' => $dstfrom,
+				   '.copyfrom_rev' => $dstrev,
+				   '.orig_copyfrom' => $from,
+				   '.orig_copyfrom_rev' => $rev,
+				 });
+    return $self->{cb_copyfrom}->($dstfrom, $dstrev)
+	if $self->{cb_copyfrom};
+    return ($dstfrom, $dstrev);
+}
+
 sub open_directory {
     my ($self, $path, $pdir, $rev, @arg) = @_;
     my $pool = $arg[-1];
+
     unless ($self->{open_nonexist}) {
 	return undef unless defined $pdir;
-	unless ($self->{cb_exist}->($path, $pool) || $self->{open_nonexist}) {
+
+	my ($basepath, $fromrev) = $self->_resolve_base($path);
+	$basepath = $path unless defined $basepath;
+
+	unless ($self->inspector->exist($basepath, $pool) || $self->{open_nonexist}) {
 	    ++$self->{skipped};
 	    $self->{notify}->flush ($path);
 	    return undef;
@@ -555,17 +688,23 @@ sub close_directory {
 
 sub _merge_file_delete {
     my ($self, $path, $rpath, $pdir, $pool) = @_;
-    return undef unless $self->{cb_localmod}->(
-		$path,
-		$self->{base_root}->file_md5_checksum ($rpath, $pool),
-		$pool);
+
+    my ($basepath, $fromrev) = $self->_resolve_base($path);
+    $basepath = $path unless defined $basepath;
+    
+    my $no_base;
+    my $md5 = $self->{base_root}->check_path ($rpath, $pool)?
+        $self->{base_root}->file_md5_checksum ($rpath, $pool)
+        : do { $no_base = 1; require Digest::MD5; Digest::MD5::md5_hex('') };
+
+    return undef unless $self->inspector->localmod ($basepath, $md5, $pool);
     return {} unless $self->{resolve};
 
     my $fh = $self->{info}{$path}->{fh} || {};
-    $fh->{base} ||= [$self->_retrieve_base($path, $pool)];
+    $fh->{base} ||= [$no_base? (tmpfile('base-')): ($self->_retrieve_base($path, $pool))];
     $fh->{new} = [tmpfile('new-')];
     $fh->{local} = [tmpfile('local-')];
-    my ($tmp) = $self->{cb_localmod}->($path, '', $pool);
+    my ($tmp) = $self->inspector->localmod($basepath, '', $pool);
     slurp_fh ( $tmp->[FH], $fh->{local}[FH]);
     seek $fh->{local}[FH], 0, 0;
     $fh->{local}[CHECKSUM] = $tmp->[CHECKSUM];
@@ -591,104 +730,164 @@ sub _merge_file_delete {
 
     return 1;
 }
-# returns undef for deleting this, a hash for partial delete.
-# returns 1 for merged delete
-# Note that empty hash means don't delete.
+
+# return a hash for partial delete
+# returns undef for deleting this
+# returns 1 for merged delete (user changed content and we leave node)
+# Note that empty hash means don't delete - conflict.
 sub _check_delete_conflict {
     my ($self, $path, $rpath, $kind, $pdir, $pool) = @_;
 
-    return $self->_merge_file_delete($path, $rpath, $pdir, $pool) if $kind == $SVN::Node::file;
+    my $localkind = $self->inspector->exist ($path, $pool);
 
-    my $dirmodified = $self->{cb_dirdelta}->($path, $self->{base_root}, $rpath);
+    # node doesn't exist in dst
+    return undef unless $localkind;
+
+    # deleting, but local node is of different type already
+    # original node could be moved to different place
+    # Editor::Rename should track the latter case
+    # XXX: prompt for resolution
+    return {} if $kind && $kind != $localkind;
+
+    return $self->_merge_file_delete ($path, $rpath, $pdir, $pool) if $localkind == $SVN::Node::file;
+
+    # TODO: checkouts may have unversioned files/dirs under the dir we are going to delete
+    # we still has no interactive resolver for this
+    return {} unless $localkind == $SVN::Node::dir;
+
+    # it's dir...
+
+    my $dirmodified = $self->inspector->dirdelta ($path, $self->{base_root}, $rpath);
     my $entries = $self->{base_root}->dir_entries ($rpath);
-    my ($torm, $modified, $merged);
+
+    my $baton = $self->{storage_baton}{$path} = $self->{storage}->open_directory (
+        $path, $self->{storage_baton}{$pdir}, $self->{cb_rev}->($path), $pool
+    );
+
+    my $torm;
     for my $name (sort keys %$entries) {
 	my ($cpath, $crpath) = ("$path/$name", "$rpath/$name");
+        my $entry = $entries->{$name};
+
 	if (my $mod = $dirmodified->{$name}) {
 	    if ($mod eq 'D') {
-		$self->{notify}->node_status ($cpath, 'd');
-		++$merged;
+                $torm->{$name} = undef;
 	    }
-	    else {
-		++$modified;
-		$self->node_conflict ($cpath);
-	    }
-	    delete $dirmodified->{$name};
+            else {
+                $torm->{$name} = $self->_check_delete_conflict ($cpath, $crpath, $entry->kind, $path, $pool);
+            }
+            delete $dirmodified->{$name};
 	}
 	else { # dir or unmodified file
-	    my $entry = $entries->{$name};
-	    $torm->{$name} = undef, next
-		if $entry->kind == $SVN::Node::file;
-
-	    if ($self->{cb_exist}->($cpath, $pool)) {
-		$torm->{$name} = $self->_check_delete_conflict
-		    ($cpath, $crpath, $SVN::Node::dir, $pdir, $pool);
-		if (ref ($torm->{$name})) {
-		    $self->node_conflict ($cpath);
-		    ++$modified;
-		}
-		if ($torm->{$name} && $torm->{$name} == 1) {
-		    ++$merged;
-		}
-	    }
-	    else {
-		$torm->{$name} = 1;
-		++$merged;
-	    }
+            $torm->{$name} = $self->_check_delete_conflict
+                ($cpath, $crpath, $entry->kind, $path, $pool);
 	}
     }
-    for my $name (keys %$dirmodified) {
-	my ($cpath, $crpath) = ("$path/$name", "$rpath/$name");
-	++$modified;
-	$self->node_conflict ($cpath);
+
+    foreach my $node (keys %$dirmodified) {
+        local $self->{tree_conflict} = 1;
+        my ($cpath, $crpath) = ("$path/$node", "$rpath/$node");
+        my $kind = $self->{base_root}->check_path ($crpath);
+        $torm->{$node} = $self->_check_delete_conflict ($cpath, $crpath, $kind, $path, $pool);
     }
-    if ($modified || $merged) {
-	# maybe leave the status to _partial delete?
-	$self->{notify}->node_status ("$path/$_", defined $torm->{$_} ? 'd' : 'D')
-	    for grep {!ref($torm->{$_})} keys %$torm;
-    }
-    return $torm if $modified;
-    return $merged ? 1 : undef;
+
+    $self->{storage}->close_directory ($baton, $pool);
+
+    return $torm;
 }
 
 sub _partial_delete {
-    my ($self, $torm, $path, $pbaton, $pool) = @_;
-    my $baton = $self->{storage}->open_directory ($path, $pbaton,
-						  $self->{cb_rev}->($path), $pool);
+    my ($self, $torm, $path, $pbaton, $pool, $no_status) = @_;
+
+    unless (ref $torm) {
+        my $s;
+        if ($torm && $torm == 1) {
+            $s = 'G';
+        } else {
+            if ($self->inspector->exist($path, $pool)) {
+                $self->{storage}->delete_entry (
+                    $path, $self->{cb_rev}->($path), $pbaton, $pool
+                );
+            }
+            $s = 'D';
+        }
+        $self->{notify}->node_status($path, $s) unless $no_status;
+        return $s;
+    } elsif (!keys %$torm) {
+        $self->node_conflict($path) unless $no_status;
+        return 'C';
+    }
+    # it's dir...
+
+    my $baton = $self->{storage}->open_directory ($path, $pbaton, $self->{cb_rev}->($path), $pool);
+    my $summary = '';
+    my @children_stats;
+    my $skip_children = 1;
     for (sort keys %$torm) {
 	my $cpath = "$path/$_";
-	if (ref $torm->{$_}) {
-	    $self->_partial_delete ($torm->{$_}, $cpath, $baton,
-				    SVN::Pool->new ($pool));
-	}
-	elsif ($self->{cb_exist}->($cpath, $pool)) {
-	    $self->{storage}->delete_entry ($cpath, $self->{cb_rev}->($cpath),
-					    $baton, $pool);
-	}
+        # check that out
+	my $status = $self->_partial_delete ($torm->{$_}, $cpath, $baton, SVN::Pool->new ($pool), 1);
+        push @children_stats, [$cpath, $status];
+        $skip_children = 0  unless $status eq 'D';
+        $summary = 'C' if $status eq 'C';
+        $summary = 'G' if !$summary && $status eq 'G';
     }
+    $summary ||= 'D';
     $self->{storage}->close_directory ($baton, $pool);
+
+    if ($summary eq 'D') {
+        if ($self->inspector->exist($path, $pool)) {
+            $self->{storage}->delete_entry ($path, $self->{cb_rev}->($path), $pbaton, $pool);
+        }
+        $self->{notify}->node_status ($path, 'D') unless $no_status;
+    }
+    elsif ($summary eq 'C') {
+        $self->node_conflict ($path) unless $no_status;
+    }
+    elsif ($summary eq 'G') {
+        $self->{notify}->node_status ($path, 'G') unless $no_status;
+    }
+    else { # really should be assert
+        $self->node_conflict ($path) unless $no_status;
+        $summary = 'C';
+    }
+
+    unless ($skip_children) {
+        foreach (@children_stats) {
+            if ($_->[1] ne 'C') {
+                $self->{notify}->node_status (@$_);
+            } else {
+                $self->node_conflict ($_->[0]);
+            }
+        }
+    }
+
+    return $summary;
 }
 
 sub delete_entry {
     my ($self, $path, $revision, $pdir, @arg) = @_;
     no warnings 'uninitialized';
     my $pool = $arg[-1];
-    return unless defined $pdir && $self->{cb_exist}->($path, $pool);
+    $pool->default;
+    my ($basepath, $fromrev) = $self->_resolve_base($path);
+    $basepath = $path unless defined $basepath;
 
-    my $rpath = $self->{base_anchor} eq '/' ? "/$path" : "$self->{base_anchor}/$path";
-    my $torm = $self->_check_delete_conflict ($path, $rpath,
-					      $self->{base_root}->check_path ($rpath), $pdir, @arg);
+    return unless defined $pdir && $self->inspector->exist($basepath);
+    my $rpath = $basepath =~ m{^/} ? $basepath :
+	$self->{base_anchor} eq '/' ? "/$basepath" : "$self->{base_anchor}/$basepath";
+    my $torm;
 
-    if (ref($torm)) {
-	$self->node_conflict ($path);
-	$self->_partial_delete ($torm, $path, $self->{storage_baton}{$pdir}, @arg);
-    } elsif( $torm && $torm == 1) {
-	$self->{notify}->node_status ($path, 'G');
-    } else {
-	$self->{storage}->delete_entry ($path, $self->{cb_rev}->($path),
-					$self->{storage_baton}{$pdir}, @arg);
-	$self->{notify}->node_status ($path, 'D');
+    # XXX: need txn-aware cb_*! for the case current path is from a
+    # copy and to be deleted - Note this might have been done, exam it.
+    {
+	# XXX: this is too evil
+	local $self->{base_root} = $self->{base_root}->fs->revision_root($fromrev) if $basepath ne $path;
+	my $kind = $self->{base_root}->check_path ($rpath);
+        $torm = $self->_check_delete_conflict ($path, $rpath, $kind, $pdir, @arg);
     }
+
+    $self->_partial_delete ($torm, $path, $self->{storage_baton}{$pdir}, @arg);
     ++$self->{changes};
 }
 
@@ -743,11 +942,12 @@ sub _merge_prop_change {
     my $rpath = $self->{base_anchor} eq '/' ? "/$path" : "$self->{base_anchor}/$path";
     my $prop;
     $prop->{new} = $_[1];
+    my ($basepath, $fromrev) = $self->{info}{$path}{baseinfo} ? @{$self->{info}{$path}{baseinfo}} : ($path);
     {
 	local $@;
 	$prop->{base} = eval { $self->{base_root}->node_prop ($rpath, $_[0], $pool) };
-	$prop->{local} = $self->{cb_exist}->($path, $pool)
-	    ? $self->{cb_localprop}->($path, $_[0], $pool) : undef;
+	$prop->{local} = $self->inspector->exist($basepath, $pool)
+	    ? $self->inspector->localprop($basepath, $_[0], $pool) : undef;
     }
     # XXX: only known props should be auto-merged with default resolver
     $pool = pop @_ if ref ($_[-1]) =~ m/^(?:SVN::Pool|_p_apr_pool_t)$/;
@@ -792,7 +992,7 @@ sub close_edit {
 	$self->{storage}->close_edit(@arg);
     }
     else {
-	print loc("Empty merge.\n");
+	$logger->warn(loc("Empty merge.")) unless $self->{notify}{quiet};
 	$self->{storage}->abort_edit(@arg);
     }
 }
@@ -807,19 +1007,6 @@ still very primitive, have to handle lots of cases
 
 =back
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
 =cut
 
 1;
