@@ -1,9 +1,59 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Editor::Patch;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 require SVN::Delta;
-our @ISA = qw(SVN::Delta::Editor);
+use base 'SVK::Editor';
 
 =head1 NAME
 
@@ -26,15 +76,6 @@ then the method name and its arguments.
 
 =cut
 
-sub baton_at {
-    my ($self, $func) = @_;
-    return -1
-	if $func eq 'set_target_revision' || $func eq 'open_root' ||
-	    $func eq 'close_edit' || $func eq 'abort_edit';
-    return 2 if $func eq 'delete_entry';
-    return $func =~ m/^(?:add|open|absent)/ ? 1 : 0;
-}
-
 sub AUTOLOAD {
     my ($self, @arg) = @_;
     my $func = our $AUTOLOAD;
@@ -52,6 +93,7 @@ sub AUTOLOAD {
     }
 
     my $ret = $func =~ m/^(?:add|open)/ ? ++$self->{batons} : undef;
+    Carp::cluck unless defined $func;
     push @{$self->{edit_tree}[$baton]}, [$ret, $func, @arg];
     return $ret;
 }
@@ -103,19 +145,5 @@ sub drive {
     SVN::Pool::apr_pool_destroy ($pool);
 }
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
 
 1;
