@@ -1,8 +1,59 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Editor::Rename;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 use base qw(SVK::Editor::Patch);
 use SVK::I18N;
+use SVK::Util 'is_path_inside';
 
 =head1 NAME
 
@@ -35,19 +86,13 @@ proper parent directory before calls are emitted to C<$next_editor>.
 
 =cut
 
-sub _path_inside {
-    my ($path, $parent) = @_;
-    return 1 if $path eq $parent;
-    return substr ($path, 0, length ($parent)+1) eq "$parent/";
-}
-
 sub rename_check {
     my ($self, $path, $nocache) = @_;
     return $self->{rename_cache}{$path}
 	if exists $self->{rename_cache}{$path};
     for (@{$self->{rename_map}}) {
 	my ($from, $to) = @$_;
-	if (_path_inside ($path, $from)) {
+	if (is_path_inside($path, $from)) {
 	    my $newpath = $path;
 	    $newpath =~ s/^\Q$from\E/$to/;
 	    $newpath = $self->rename_check ($newpath, 1);
@@ -80,8 +125,7 @@ sub AUTOLOAD {
     return if $func =~ m/^[A-Z]+$/;
     my $baton_at = $self->baton_at ($func);
     my ($renamed, $renamed_anchor);
-
-    if ($baton_at == 1) {
+    if ($baton_at > 0) {
 	my $newpath = $self->rename_check ($arg[0]);
 	if ($newpath ne $arg[0]) {
 	    ++$renamed;
@@ -100,10 +144,10 @@ sub AUTOLOAD {
     my $sfunc = "SUPER::$func";
     my $ret = $self->$sfunc (@arg);
 
-    $self->{renamed}[$ret]++ if $renamed;
+    $self->{renamed}[$ret]++ if $renamed && $ret;
 
     if ($renamed_anchor) {
-	$self->{renamed_anchor}[$ret] = $self->{edit_tree}[$arg[1]][-1]
+	push @{$self->{renamed_anchor}}, $self->{edit_tree}[$arg[$baton_at]][-1];
     }
     else {
 	$self->{opened_baton}{$arg[0]} = [$ret, $arg[1]]
@@ -124,7 +168,8 @@ sub open_parent {
 
     ++$self->{batons};
 
-    if ($self->{cb_exist} && !$self->{cb_exist}->($parent)) {
+    # XXX: If inspector is always there, then the first check isn't necessary.
+    if ($self->{inspector} && !$self->{inspector}->exist($parent)) {
 	unshift @{$self->{edit_tree}[$pbaton]},
 	    [$self->{batons}, 'add_directory', $parent, $ppbaton, undef, -1];
     }
@@ -148,6 +193,10 @@ sub adjust_anchor {
     @$entry = [];
 }
 
+sub adjust_last_anchor {
+    $_[0]->adjust_anchor($_[0]{edit_tree}[0][-1]);
+}
+
 sub _insert_entry {
     my ($self, $anchor, $entry) = @_;
     # move the call to a proper place.
@@ -163,28 +212,21 @@ sub _insert_entry {
 sub close_edit {
     my $self = shift;
     $self->SUPER::close_edit (@_);
-    for (0..$#{$self->{renamed_anchor}}) {
-	next unless defined $self->{renamed_anchor}[$_];
-	$self->adjust_anchor ($self->{renamed_anchor}[$_]);
+    for (@{$self->{renamed_anchor}}) {
+	$self->adjust_anchor($_);
     }
     # XXX: addition phase here to trim useless opens.
     $self->drive ($self->{editor});
 #SVN::Delta::Editor->new (_debug => 1, _editor => [$self->{editor}]));
 }
 
-=head1 AUTHORS
+# Make sure driven editor aborts too.
+sub abort_edit {
+    my $self = shift;
+    $self->SUPER::abort_edit (@_);
+    my $r = $self->{editor}->abort_edit(@_);
+    return $r;
+}
 
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
 
 1;
