@@ -1,8 +1,59 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Patch;
 use strict;
 use SVK::I18N;
 use SVK::Util qw( read_file write_file );
 use SVK::Version;  our $VERSION = $SVK::VERSION;
+use SVK::Logger;
 
 =head1 NAME
 
@@ -37,7 +88,6 @@ info and anchor for the patch to be applied.
 =cut
 
 use SVK::Editor::Patch;
-use SVK::Util qw(find_svm_source find_local_mirror resolve_svm_source);
 use SVK::Merge;
 use SVK::Editor::Diff;
 use SVK::Target::Universal;
@@ -58,10 +108,9 @@ sub new {
     my $self = bless { name   => $name,
 		       level  => 0,
 		       _xd    => $xd,
-		       _depot => $depotname,
 		       _source => $src,
 		       _target => $dst }, $class;
-    (undef, undef, $self->{_repos}) = $self->{_xd}->find_repos ("/$self->{_depot}/", 1);
+    $self->{_depot} = $self->{_xd}->find_depot($depotname);
     $self->{source} = $self->{_source}->universal if $self->{_source};
     $self->{target} = $self->{_target}->universal;
     return $self;
@@ -98,14 +147,14 @@ sub load {
 
     my ($self) = thaw (uncompress (decode_base64 ( $content ) ) );
     $self->{_xd} = $xd;
-    $self->{_depot} = $depot;
+    $self->{_depot} = $self->{_xd}->find_depot($depot);
 
     my $not_applicable;
 
     for (qw/source target/) {
 	next unless $self->{$_};
-	my $tmp = $self->{"_$_"} = $self->{$_}->local ($xd, $depot) or next;
-	$tmp = $tmp->new (revision => undef);
+	my $tmp = $self->{"_$_"} = $self->{$_}->local($self->{_depot}) or next;
+	$tmp = $tmp->new->refresh_revision;
 	if (wantarray) {
 	    eval { $tmp->normalize };
 	    $not_applicable = $@
@@ -114,9 +163,8 @@ sub load {
 	    $tmp->normalize;
 	}
 	$self->{"_${_}_updated"} = 1
-	    if $tmp->{revision} > $self->{"_$_"}->{revision};
+	    if $tmp->{revision} > $self->{"_$_"}->revision;
     }
-    (undef, undef, $self->{_repos}) = $self->{_xd}->find_repos ("/$self->{_depot}/", 1);
     return wantarray ? ($self, $not_applicable) : $self;
 }
 
@@ -161,31 +209,28 @@ sub editor {
 sub _path_attribute_text {
     my ($self, $type, $no_label) = @_;
     # XXX: check if source / target is updated
-    my ($local, $mirrored, $updated);
+    my ($local, $m, $updated);
     if (my $target = $self->{"_$type"}) {
-	if ($target->{repos}->fs->get_uuid eq $self->{$type}{uuid}) {
+	if ($target->repos->fs->get_uuid eq $self->{$type}{uuid}) {
 	    ++$local;
 	}
 	else {
-            my ($repos, $path) = @$target{qw/repos path/};
-            (undef, $mirrored) = resolve_svm_source(
-                $repos, find_svm_source($repos, $path)
-            );
+	    $m= $target->is_mirrored or die;
 	}
     }
     my $label = $no_label ? '' : join(
 	' ', '',
 	($local ? '[local]' : ()),
-	($mirrored ? '[mirrored]' : ()),
+	($m? '[mirrored]' : ()),
 	($self->{"_${type}_updated"} ? '[updated]' : ())
     );
-    $label .= "\n        ($mirrored->{source})" if $mirrored;
+    $label .= "\n        (".$m->url.")" if $m;
     return $label;
 }
 
 sub view {
     my ($self, $output) = @_;
-    my $fs = $self->{_repos}->fs;
+    my $fs = $self->{_depot}->repos->fs;
 
     die loc("Target not local nor mirrored, unable to view patch.\n")
 	unless $self->{_target};
@@ -212,14 +257,7 @@ sub view {
     my $anchor = $self->{_target}->path;
     $self->editor->drive
 	( SVK::Editor::Diff->new
-	  ( cb_basecontent => sub { my ($path, $pool) = @_;
-				    my $base = $baseroot->file_contents ("$anchor/$path", $pool);
-				    return $base;
-				},
-	    cb_baseprop => sub { my ($path, $pname, $pool) = @_;
-				 return $baseroot->node_prop ("$anchor/$path", $pname, $pool);
-			     },
-	    oldtarget => $self->{_target}, oldroot => $baseroot,
+	  ( base_target => $self->{_target}, base_root => $baseroot,
 	    llabel => "revision $self->{target}{rev}",
 	    rlabel => "patch $self->{name} level $self->{level}",
 	    external => $ENV{SVKDIFF},
@@ -229,8 +267,8 @@ sub view {
 
 sub apply {
     my ($self, $check_only) = @_;
-    my $commit = SVK::Command->get_cmd ('commit', $self->{_xd});
-    my $target = $self->{_target};
+    my $commit = SVK::Command->get_cmd ('commit', xd => $self->{_xd});
+    my $target = $self->{_target}->new->refresh_revision;
     $commit->{message} = "Apply $self->{name}\@$self->{level}";
     $commit->{check_only} = $check_only;
     $self->apply_to ($target, $commit->get_editor ($target));
@@ -248,9 +286,9 @@ sub apply_to {
 	  anchor => $target->path,
 	  target => '',
 	  send_fulltext => !$cb{patch} && !$cb{mirror},
-	  ($target->{copath}
+	  ($target->isa('SVK::Path::Checkout')
 	      ? (notify => SVK::Notify->new_with_report
-		    ($target->{report}, $target, 1))
+		    ($target->report, $target, 1))
 	      : ()
 	  ),
 	  %cb,
@@ -266,16 +304,16 @@ sub update {
 	unless $self->{_target};
 
     return unless $self->{_target_updated};
-    my $target = $self->{_target}->new (revision => undef);
+    my $target = $self->{_target}->new->refresh_revision;
     $target->normalize;
     my $patch = SVK::Editor::Patch->new;
     my $conflict;
 
     if ($conflict = $self->apply_to ($target, $patch, patch => 1,
 				     SVK::Editor::Merge->cb_for_root
-				     ($target->root, $target->path, $target->{revision}))) {
+				     ($target->root, $target->path, $target->revision))) {
 
-	print loc("Conflicts.\n");
+	$logger->error(loc("Conflicts."));
 	return $conflict;
     }
 
@@ -290,19 +328,19 @@ sub regen {
     my $target = $self->{_target}
 	or die loc("Target not local nor mirrored, unable to regen patch.\n");
     unless ($self->{level} == 0 || $self->{_source_updated}) {
-	print loc("Source of patch %1 not updated or not local, no need to regen patch.\n", $self->{name});
+	$logger->warn(loc("Source of patch %1 not updated or not local, no need to regen patch.", $self->{name}));
 	return;
     }
-    my $source = $self->{_source}->new (revision => undef);
+    my $source = $self->{_source}->new->refresh_revision;
     $source->normalize;
-    my $merge = SVK::Merge->auto (repos => $self->{_repos}, xd => $self->{_xd},
+    my $merge = SVK::Merge->auto (repos => $self->{_depot}->repos, xd => $self->{_xd},
 				  src => $source, dst => $target);
     my $conflict;
     my $patch = SVK::Editor::Patch->new;
     # XXX: handle empty
     unless ($conflict = $merge->run ($patch,
 				     SVK::Editor::Merge->cb_for_root
-				     ($target->root, $target->path, $target->{revision}))) {
+				     ($target->root, $target->path, $target->revision))) {
 	$self->{log} = $merge->log (1);
 	++$self->{level};
 	$self->{_source} = $source;
@@ -343,6 +381,7 @@ sub commit_editor {
 package SVK::Patch::CommitEditor;
 use base qw(SVK::Editor::Patch);
 use SVK::I18N;
+use SVK::Logger;
 
 sub close_edit {
     my $self = shift;
@@ -353,22 +392,8 @@ sub close_edit {
     ++$patch->{level};
     $patch->store ($filename);
     return if $filename eq '-';
-    print loc ("Patch %1 created.\n", $patch->{name});
+    $logger->warn(loc ("Patch %1 created.", $patch->{name}));
 }
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
 
 1;
