@@ -1,3 +1,53 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Resolve;
 use strict;
 use SVK::I18N;
@@ -6,6 +56,7 @@ use SVK::Util qw(
     read_file can_run is_executable bsd_glob
 );
 use File::Copy ();
+use SVK::Logger;
 
 use constant Actions => {qw(
     a   accept      e   edit        d   diff
@@ -122,12 +173,12 @@ sub merge {
     ) : $self->get_resolver);
 
     if (!$resolver) {
-        print loc("Cannot launch an external merge tool for %1.\n", $path);
+        $logger->error(loc("Cannot launch an external merge tool for %1.", $path));
         return;
     }
 
     # maybe some message here
-    print loc("Invoking merge tool '%1' for %2.\n", $resolver->name, $path);
+    $logger->info(loc("Invoking merge tool '%1' for %2.", $resolver->name, $path));
 
     if ($resolver->run_resolver($cmd, @args)) {
         $self->{has_conflict} = (
@@ -165,7 +216,7 @@ sub filter_conflict {
 
     my $state = '';
     my $newline = "[\r\n|\n\r?]";
-    my $in  = qr/^[>=]{4} (YOUR|ORIGINAL|THEIR) VERSION $self->{path} $self->{marker}$newline$/;
+    my $in  = qr/^[>=]{4} (YOUR|ORIGINAL|THEIR) VERSION $self->{path}(?: \(.*\))? $self->{marker}$newline$/;
     my $out = qr/<<<< $self->{marker}$newline/;
 
     while (<$ifh>) {
