@@ -1,3 +1,53 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Util;
 use strict;
 require Exporter;
@@ -9,31 +59,34 @@ our @EXPORT_OK = qw(
 
     get_encoding get_encoder from_native to_native
 
-    find_local_mirror find_svm_source resolve_svm_source traverse_history
-    find_prev_copy
+    find_svm_source traverse_history
 
     read_file write_file slurp_fh md5_fh bsd_glob mimetype mimetype_is_text
+    is_binary_file
 
     abs_path abs2rel catdir catfile catpath devnull dirname get_anchor 
     move_path make_path splitpath splitdir tmpdir tmpfile get_depot_anchor
     catdepot abs_path_noexist 
 
-    is_symlink is_executable is_uri can_run
+    is_symlink is_executable is_uri can_run is_path_inside
+
+    str2time time2str reformat_svn_date
+
+    find_dotsvk
 );
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 
 use Config ();
+use SVK::Logger;
 use SVK::I18N;
 use SVN::Core;
-use SVN::Ra;
 use autouse 'Encode'            => qw(resolve_alias($) decode encode);
 use File::Glob qw(bsd_glob);
 use autouse 'File::Basename' 	=> qw(dirname);
 use autouse 'File::Spec::Functions' => 
                                qw(catdir catpath splitpath splitdir tmpdir);
-
-use Class::Autouse qw( File::Type );
+use autouse 'List::Util'        => qw( max(@) );
 
 
 =head1 NAME
@@ -200,7 +253,7 @@ sub edit_file {
 			: DEFAULT_EDITOR; # fall back to something
     my @editor = split (/ /, $editor);
 
-    print loc("Waiting for editor...\n");
+    $logger->info(loc("Waiting for editor..."));
 
     # XXX: check $?
     system {$editor[0]} (@editor, $file) and die loc("Aborted: %1\n", $!);
@@ -325,72 +378,13 @@ sub to_native {
     return;
 }
 
-=head2 Mirror Handling
-
-=head3 find_local_mirror ($repos, $uuid, $path, $rev)
-
-XXX Undocumented
-
-=cut
-
-sub find_local_mirror {
-    my ($repos, $uuid, $path, $rev) = @_;
-    my $myuuid = $repos->fs->get_uuid;
-    return unless HAS_SVN_MIRROR && $uuid ne $myuuid;
-    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
-    return ("$m->{target_path}$mpath",
-	    $rev ? $m->find_local_rev ($rev) : $rev) if $m;
-}
-
-=head3 find_svm_source ($repos, $path, $rev)
-
-XXX Undocumented
-
-=cut
-
-sub find_svm_source {
+sub find_svm_source { # DEPRECATED: use SVK::Path->universal, only used in SVK::Command now.
     my ($repos, $path, $rev) = @_;
-    my $fs = $repos->fs;
-    $rev ||= $fs->youngest_rev;
-    my $root = $fs->revision_root ($rev);
-    my ($uuid, $m, $mpath);
-
-    if (HAS_SVN_MIRROR) {
-	($m, $mpath) = SVN::Mirror::is_mirrored ($repos, $path);
-    }
-
-    if ($m) {
-	# XXX: we should normalize $rev before calling find_svm_source
-	$rev = ($root->node_history($path)->prev(0)->location)[1]
-	    unless $rev == $root->node_created_rev ($path);
-	$rev = $m->find_remote_rev ($rev);
-	$path =~ s/\Q$mpath\E$//;
-	$uuid = $m->{source_uuid};
-	$path = $m->{source_path}.$mpath;
-	$path ||= '/';
-    }
-    else {
-	$uuid = $fs->get_uuid;
-	$rev = ($root->node_history ($path)->prev (0)->location)[1];
-    }
-
-    return ($uuid, $path, $rev);
-}
-
-=head3 resolve_svm_source ($repos, $uuid, $path)
-
-XXX Undocumented
-
-=cut
-
-sub resolve_svm_source {
-    my ($repos, $uuid, $path) = @_;
-    my $myuuid = $repos->fs->get_uuid;
-    return ($path) if ($uuid eq $myuuid);
-    return unless HAS_SVN_MIRROR;
-    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
-    return unless $m;
-    return ("$m->{target_path}$mpath", $m);
+    my $t = SVK::Path->real_new({ depot => SVK::Depot->new({repos => $repos}),
+                                  path => $path, revision => $rev });
+    $t->refresh_revision unless $rev;
+    my $u = $t->universal;
+    return map { $u->$_ } qw(uuid path rev);
 }
 
 =head2 File Content Manipulation
@@ -475,22 +469,20 @@ is missing on the system.
 =cut
 
 sub mimetype {
-    my $fh = shift;
+    my ($filename) = @_;
+    my $mm if 0;  # C<state $mm>, yuck
 
-    return 'text/plain' if -z $fh;
-
-    binmode($fh);
-    read $fh, my $data, 16*1024 or return undef;
-
-    my $type = File::Type->checktype_contents($data);
-
-    # On fallback, use the same logic as File::MimeInfo to detect text
-    if ($type eq 'application/octet-stream') {
-        substr($data, 0, 32) =~ m/[\x00-\x07\x0B\x0E-\x1A\x1C-\x1F]/
-            or return 'text/plain';
+    # find an implementation module if necessary
+    if ( !$mm ) {
+        my $module = $ENV{SVKMIME} || 'Internal';
+        $module =~ s/:://;
+        $module = "SVK::MimeDetect::$module";
+        eval "require $module";
+        die $@ if $@;
+        $mm = $module->new();
     }
 
-    return $type;
+    return $mm->checktype_filename($filename);
 }
 
 =head3 mimetype_is_text ($mimetype)
@@ -514,6 +506,21 @@ sub mimetype_is_text {
                          |image/x-x(?:bit|pix)map)$}x;
 }
 
+=head3 is_binary_file ($filename OR $filehandle)
+
+Returns true if the given file or filehandle contains binary data.  Otherwise,
+returns false.
+
+=cut
+
+sub is_binary_file {
+    my ($file) = @_;
+
+    # let Perl do the hard work
+    return 1 if -f $file && !-T _;  # !-T handles empty files correctly
+    return;
+}
+
 =head2 Path and Filename Handling
 
 =head3 abspath ($path)
@@ -579,11 +586,11 @@ sub abs_path_noexist {
 Replace the base directory in the native pathname to another base directory
 and return the result.
 
-If the pathname is not under C<$old_basedir>, it is not unmodified.
+If the pathname is not under C<$old_basedir>, it is returned unmodified.
 
 If C<$new_basedir> is an empty string, removes the old base directory but
-keeps the trailing slash.  If C<$new_basedir> is C<undef>, also removes
-the trailing slash.
+keeps the leading slash.  If C<$new_basedir> is C<undef>, also removes
+the leading slash.
 
 By default, the return value of this function will use C<$SEP> as its
 path separator.  Setting C<$sep> to C</> will turn native path separators
@@ -817,12 +824,23 @@ sub move_path {
         File::Copy::move ($source => $target) and return;
     }
 
-    print loc(
-        "Cannot rename %1 to %2; please move it manually.\n",
+    $logger->error(loc(
+        "Cannot rename %1 to %2; please move it manually.",
         catfile($source), catfile($target),
-    );
+    ));
 }
 
+=head3 traverse_history (root => $fs_root, path => $path,
+    cross => $cross, callback => $cb($path, $revision))
+
+Traverse the history of $path in $fs_root backwards until the first
+copy, unless $cross is true.  We do cross renames regardless of the
+value of $cross being non-zero, but not -1.  We invoke $cb for each
+$path, $revision we encounter.  If cb returns a nonzero value we stop
+traversing as well.
+
+=cut
+
 sub traverse_history {
     my %args = @_;
 
@@ -830,129 +848,148 @@ sub traverse_history {
     my $new_pool = SVN::Pool->new;
     my $spool = SVN::Pool->new_default;
 
-    my $hist = $args{root}->node_history ($args{path}, $old_pool);
-    my $rv;
+    my ($root, $path) = @args{qw/root path/};
+    # If the root is txn root, get a similar one.
+    # XXX: We actually want to move this to SVK::Path::, and
+    # svk::checkout should respect copies on checkout
+    if ($root->can('txn') && $root->txn) {
+	($root, $path) = $root->get_revision_root
+	    ($path, $root->txn->base_revision );
+    }
 
-    while ($hist = $hist->prev(($args{cross} || 0), $new_pool)) {
-        $rv = $args{callback}->($hist->location ($new_pool));
-        last if !$rv;
+    my $hist = $root->node_history ($path, $old_pool);
+    my $rv;
+    my $revision;
+
+    while (1) {
+        my $ohist = $hist;
+        $hist = $hist->prev(max(0, $args{cross} || 0), $new_pool);
+        if (!$hist) {
+            last if $args{cross};
+            last unless $hist = $ohist->prev((1), $new_pool);
+            # We are not supposed to cross copies, ($path,$revision)
+            # refers to a node in $ohist that is a copy and that has a
+            # prev if we ask svn to traverse copies.
+            # Let's find out if the copy was actually a rename instead
+            # of a copy.
+            my $root = $root->fs->revision_root($revision, $spool);
+            my $frompath;
+            my $fromrev = -1;
+            # We know that $path was a real copy and it that it has a
+            # prev, so find the node from which it was copied.
+            do {
+                ($fromrev, $frompath) = $root->copied_from($path, $spool);
+            } until ($fromrev >= 0 || !($path =~ s{/[^/]*$}{}));
+            die "Assertion failed: $path in $revision isn't a copy."
+                if $fromrev < 0;
+            # Ok, $path in $root was a copy of ($frompath,$fromrev).
+            # If $frompath was deleted in $root then the copy was really
+            # a rename.
+            my $entry = $root->paths_changed($spool)->{$frompath};
+            last unless $entry &&
+                $entry->change_kind == $SVN::Fs::PathChange::delete;
+
+            # XXX Do we need to worry about a parent of $frompath having
+            # been deleted instead?  If so the 2 lines below might work as
+            # an alternative, to the previous 3 lines.  However this also
+            # treats a delete followed by a copy of an older revision in
+            # two separate commits as a rename, which technically it's not.
+            #last unless $root->check_path($frompath, $spool) ==
+            #    $SVN::Node::none;
+        }
+        ($path, $revision) = $hist->location ($new_pool);
         $old_pool->clear;
-	$spool->clear;
+        $rv = $args{callback}->($path, $revision);
+        last if !$rv;
+        $spool->clear;
         ($old_pool, $new_pool) = ($new_pool, $old_pool);
     }
 
     return $rv;
 }
 
-=head3 find_prev_copy ($fs, $rev)
+sub reformat_svn_date {
+    my ($format, $svn_date) = @_;
+    return time2str($format, str2time($svn_date));
+}
 
-Find the revision of the nearest copy in a repository that is less or
-equal to C<$rev>.  Returns the found revision number, and a hash of
-arrayref that contains copied paths and its source found in that
-revision.
+sub str2time {
+    require Time::Local;
+    my ($year, $month, $day, $hh, $mm, $ss) = split /[-T:]/, $_[0];
+    $year -= 1900;
+    $month--;
+    chop($ss);  # remove the 'Z'
+    my $zone = 0;  # UTC
+
+    my @lt = localtime(time);
+
+    my $frac = $ss - int($ss);
+    $ss = int $ss;
+
+    for ( $year, $month, $day, $hh, $mm, $ss ) {
+        return undef unless defined($_) 
+    }
+    return undef
+      unless ( $month <= 11
+        && $day >= 1
+        && $day <= 31
+        && $hh <= 23
+        && $mm <= 59
+        && $ss <= 59 );
+
+    my $result;
+
+    $result = eval {
+        local $SIG{__DIE__} = sub { };    # Ick!
+        Time::Local::timegm( $ss, $mm, $hh, $day, $month, $year );
+    };
+    return undef
+        if !defined $result
+        or $result == -1
+        && join( "", $ss, $mm, $hh, $day, $month, $year ) ne "595923311169";
+
+    return $result + $frac;
+}
+
+sub time2str {
+    my ($format, $time) = @_;
+    require POSIX;
+    return POSIX::strftime($format, localtime($time) );
+}
+
+
+sub find_dotsvk {
+    require Cwd;
+    require Path::Class;
+
+    my $p = Path::Class::Dir->new( Cwd::cwd() );
+
+    my $prev = "not $p";
+    my $found = q{};
+    while ( $p && $p ne $prev && -r $p ) {
+	$prev = $p;
+	my $svk = $p->subdir('.svk');
+	return $svk if -e $svk;
+	$p = $p->parent();
+    }
+
+    return
+}
+
+=head3 is_path_inside($path, $parent)
+
+Returns true if unix path C<$path> is inside C<$parent>.
+If they are the same, return true as well.
 
 =cut
 
-sub _copies_in_root {
-    my ($root) = @_;
-    my $copies;
-    my $changed = $root->paths_changed;
-    my $pool = SVN::Pool->new_default;
-    for (keys %$changed) {
-	$pool->clear;
-	next if $changed->{$_}->change_kind == $SVN::Fs::PathChange::delete;
-	my ($copyfrom_rev, $copyfrom_path) = $root->copied_from ($_);
-	$copies->{$_} = [$copyfrom_rev, $copyfrom_path]
-	    if defined $copyfrom_path;
-    }
-    return $copies;
-}
-
-sub find_prev_copy {
-    my ($fs, $endrev, $ppool) = @_;
-    my $pool = SVN::Pool->new_default;
-    # hold this resulting root in the subpool of ppool.
-    my $spool = $ppool ? SVN::Pool::create ($$ppool) : $pool;
-    my ($rev, $startrev) = ($endrev, $endrev);
-    my ($root, $copy);
-    while ($rev > 0) {
-	$pool->clear;
-	SVN::Pool::apr_pool_clear ($spool) if $ppool;
-	if (defined (my $cache = $fs->revision_prop ($rev, 'svk:copy_cache_prev'))) {
-	    $startrev = $rev + 1;
-	    $rev = $cache;
-	    last if $rev == 0;
-	}
-	$root = $fs->revision_root ($rev, $spool);
-	if ($copy = _copies_in_root ($root)) {
-	    last;
-	}
-	--$rev; --$startrev;
-    }
-    $fs->change_rev_prop ($_, 'svk:copy_cache_prev', $rev), $pool->clear
-	for $startrev..$endrev;
-    return unless $rev;
-    return ($root, $copy);
-}
-
-
-# this is the cached and faster version of svn::mirror::has_local,
-# which should be deprecated eventually.
-my %mirror_cached;
-
-sub _list_mirror_cached {
-    my $repos = shift;
-    my $rev = $repos->fs->youngest_rev;
-    delete $mirror_cached{$repos}
-	unless ($mirror_cached{$repos}{rev} || -1) == $rev;
-    return %{$mirror_cached{$repos}{hash}}
-	if exists $mirror_cached{$repos};
-    my %mirrored = map {
-	my $m = SVN::Mirror->new( target_path => $_,
-				  repos => $repos,
-				  pool => SVN::Pool->new,
-				  get_source => 1 );
-	local $@;
-	eval { $m->init };
-	$@ ? () : ($_ => join(':', $m->{source_uuid}, $m->{source_path}))
-    } SVN::Mirror::list_mirror($repos);
-
-    $mirror_cached{$repos} = { rev => $rev, hash => \%mirrored};
-    return %mirrored;
-}
-
-sub _has_local {
-    my ($repos, $spec) = @_;
-    my %mirrored = _list_mirror_cached($repos);
-    while (my ($path, $mspec) = each(%mirrored)) {
-	my $mpath = $spec;
-	next unless $mpath =~ s/^\Q$mspec\E//;
-	my $m = SVN::Mirror->new (target_path => $path,
-				  repos => $repos,
-				  pool => SVN::Pool->new,
-				  get_source => 1);
-	$m->init;
-	$mpath = '' if $mpath eq '/';
-	return ($m, $mpath);
-    }
-    return;
+sub is_path_inside {
+    my ($path, $parent) = @_;
+    return 1 if $path eq $parent;
+    return substr ($path, 0, length ($parent)+1) eq "$parent/";
 }
 
 1;
 
 __END__
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
